package com.mx.dla.dda.admin.catalogos.tipoCambioAddOns.daos;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.mx.dla.dda.admin.catalogos.tipoCambioAddOns.dtos.PaisAddOnsVDTO;
import com.mx.dla.dda.admin.catalogos.tipoCambioAddOns.dtos.TipoCambioAddOnsDTO;
import com.mx.dla.dda.admin.catalogos.tipoCambioAddOns.dtos.TipoCambioPaisMesAddOnsDTO;

public interface TipoCambioAddOnsDAO {
	
	public void creaTipoCambio(TipoCambioAddOnsDTO param);

	public void actualizaTipoCambio(TipoCambioAddOnsDTO param);

	public List<TipoCambioAddOnsDTO> consultaTiposCambio();
	
	public List<PaisAddOnsVDTO> obtenerPaisesAddOnXEstudio(@Param("idEstudio")Long idEstudio);
	
	public List<TipoCambioAddOnsDTO> obtnerMesTiposXPais(@Param("anio") Integer anio, @Param("idEstudio") Integer idEstudio, @Param("idPais") Long idPais);
	
	
	public List<TipoCambioPaisMesAddOnsDTO> obtnerPaisMes(@Param("idEstudio") Long idEstudio,@Param("anio") Integer anio);
	
	public PaisAddOnsVDTO obtenerPaisXSigla(@Param("sigla") String sigla);
	
	
	public Integer verificarExistenciaTipoCambioAddOn(@Param("idRelacion") Long idRelacion, @Param("mes") String mes);
}
